@@ -16,17 +16,32 @@ EdgesType = T.TypeVar("EdgesType", bound=BaseModel)
 
 ResolverType = T.TypeVar("ResolverType", bound="Resolver")
 
+BaseModelType = T.TypeVar("BaseModelType", bound=BaseModel)
+
+
+def get_fields_and_nested_fields(cls: T.Type[BaseModelType]) -> T.List[str]:
+    field_names: T.List[str] = []
+    for field_name, model_field in cls.__fields__.items():
+        t = model_field.type_
+        if issubclass(t, BaseModel):
+            field_names.append(
+                f"{field_name}{{{','.join(get_fields_and_nested_fields(t))}}}"
+            )
+        else:
+            field_names.append(field_name)
+    return [*field_names, "__typename"]
+
 
 class Resolver(BaseModel, T.Generic[NodeType]):
     node: T.ClassVar[T.Type[NodeType]]
 
-    get_params: GetParamsType
     query_params: QueryParamsType
     edges: EdgesType
 
     def gql_fields_str(self) -> str:
         """This does not include the top level..."""
-        fields = [*self.node.__fields__.keys(), "__typename"]
+        # TODO this currently just gets all nested fields -> change to make it get edges intelligently
+        fields = get_fields_and_nested_fields(self.node)
         for resolver_name in self.edges.__fields__.keys():
             resolver: T.Optional[Resolver] = getattr(self.edges, resolver_name, None)
             if resolver:
@@ -39,10 +54,12 @@ class Resolver(BaseModel, T.Generic[NodeType]):
 
     def make_get_query_str(self, kwargs_d: dict) -> str:
         kwargs = {k: v for k, v in kwargs_d.items() if v is not None}
+        # this is not necessarily true
         if not kwargs:
-            raise GQLException(
-                f".get requires one field to be given of {list(kwargs_d.keys())}"
-            )
+            pass  # for agg queries (for one)
+            # raise GQLException(
+            #     f".get requires one field to be given of {list(kwargs_d.keys())}"
+            # )
         inner_params = ",".join(
             [
                 f"{field_name}: {json.dumps(jsonable_encoder(val))}"
@@ -55,11 +72,17 @@ class Resolver(BaseModel, T.Generic[NodeType]):
 
     @classmethod
     def get_query_name(cls) -> str:
-        return f"get{cls.node.Dgraph.typename}"
+        function_name = cls.node.GQL.get_function_name
+        if not function_name:
+            raise GQLException(f"No get function found for {cls.node.GQL.typename}!")
+        return function_name
 
     @classmethod
     def query_query_name(cls) -> str:
-        return f"query{cls.node.Dgraph.typename}"
+        function_name = cls.node.GQL.query_function_name
+        if not function_name:
+            raise GQLException(f"No query function found for {cls.node.GQL.typename}!")
+        return function_name
 
     def make_query_query_str(self) -> str:
         s = f"{{ {self.query_query_name()}{self.params_and_fields()} }}"
@@ -67,39 +90,52 @@ class Resolver(BaseModel, T.Generic[NodeType]):
         return s
 
     def make_add_query_str(self) -> str:
-        s = f"{{ {self.node.Dgraph.payload_node_name}{self.params_and_fields()} }}"
+        s = f"{{ {self.node.GQL.payload_node_name}{self.params_and_fields()} }}"
         return s
 
     def make_add_mutation_str(self) -> str:
-        typename = self.node.Dgraph.typename
+        typename = self.node.GQL.typename
+        add_model_name = self.node.GQL.add_model.__name__
+        add_function_name = self.node.GQL.add_function_name
+        if not (self.node.GQL.add_model and add_function_name):
+            raise GQLException("Add model or add function is missing")
         s = f"""
-            mutation Add{typename}($input: [Add{typename}Input!]!, $upsert: Boolean) {{
-                add{typename}(input: $input, upsert: $upsert) {self.make_add_query_str()}
+            mutation Add{typename}($input: [{add_model_name}!]!, $upsert: Boolean) {{
+                {add_function_name}(input: $input, upsert: $upsert) {self.make_add_query_str()}
             }}
         """
         return s
 
     def make_update_mutation_str(self) -> str:
-        typename = self.node.Dgraph.typename
+        typename = self.node.GQL.typename
+        patch_model_name = self.node.GQL.patch_model.__name__
+        update_function_name = self.node.GQL.update_function_name
+        if not (self.node.GQL.patch_model and update_function_name):
+            raise GQLException("Update model or update function is missing")
         s = f"""
-            mutation Update{typename}($set: {typename}Patch, $remove: {typename}Patch, $filter: {typename}Filter!) {{
-                update{typename}(input: {{filter: $filter, set: $set, remove: $remove}}) {self.make_add_query_str()}
+            mutation Update{typename}($set: {patch_model_name}, $remove: {patch_model_name}, $filter: {typename}Filter!) {{
+                {update_function_name}(input: {{filter: $filter, set: $set, remove: $remove}}) {self.make_add_query_str()}
             }}
         """
         return s
 
     def make_delete_mutation_str(self) -> str:
-        typename = self.node.Dgraph.typename
+        typename = self.node.GQL.typename
+
+        delete_function_name = self.node.GQL.delete_function_name
+        if not delete_function_name:
+            raise GQLException("Delete function is missing")
+
         s = f"""
             mutation Delete{typename}($filter: {typename}Filter!) {{
-                delete{typename}(filter: $filter) {self.make_add_query_str()}
+                {delete_function_name}(filter: $filter) {self.make_add_query_str()}
             }}
         """
         return s
 
     async def query(self) -> T.List[NodeType]:
         s = self.make_query_query_str()
-        res = await execute.gql(query_str=s, url=self.node.Dgraph.url)
+        res = await execute.gql(query_str=s, url=self.node.GQL.url)
         lst: T.List[dict] = res["data"][self.query_query_name()]
         return [self.parse_obj_nested(d) for d in lst]
 
@@ -126,7 +162,7 @@ class Resolver(BaseModel, T.Generic[NodeType]):
 
     async def _get(self, kwargs_d: dict) -> T.Optional[NodeType]:
         s = self.make_get_query_str(kwargs_d=kwargs_d)
-        res = await execute.gql(query_str=s, url=self.node.Dgraph.url)
+        res = await execute.gql(query_str=s, url=self.node.GQL.url)
         obj = res["data"][self.get_query_name()]
         if obj:
             return self.parse_obj_nested(obj)
@@ -137,24 +173,10 @@ class Resolver(BaseModel, T.Generic[NodeType]):
         d = {}
         subs = Resolver.__subclasses__()
         for sub in subs:
-            typename = sub.node.Dgraph.typename
+            typename = sub.node.GQL.typename
             if typename in d:
                 raise GQLException(
                     f"Two Resolvers share the typename {typename}: ({sub.__name__}, {d[typename].__name__})"
                 )
             d[typename] = sub
         return d
-
-    """
-    def get(self, get_params: GetParamsType) -> T.Optional[NodeType]:
-        return self._get(kwargs_d=get_params.dict())
-
-    async def get_async(self, get_params: GetParamsType) -> T.Optional[NodeType]:
-        return await self._get_async(kwargs_d=get_params.dict())
-
-    def gerror(self, get_params: GetParamsType) -> NodeType:
-        node = self.get(get_params=get_params)
-        if not node:
-            raise GQLException(f"Node with {get_params=} was None")
-        return node
-    """
